@@ -33,7 +33,7 @@ from atlasapi.measurements import AtlasMeasurementTypes, AtlasMeasurementValue,
     OptionalAtlasMeasurement
 from atlasapi.events import atlas_event_factory, ListOfEvents
 import logging
-from typing import Union, Iterable, Set, BinaryIO
+from typing import Union, Iterable, Set, BinaryIO, Generator, Iterator
 from atlasapi.errors import ErrAtlasUnauthorized
 from atlasapi.alerts import Alert
 from time import time
@@ -41,6 +41,7 @@ from atlasapi.whitelist import WhitelistEntry
 from atlasapi.maintenance_window import MaintenanceWindow, Weekdays
 from atlasapi.lib import AtlasLogNames, LogLine
 from requests import get
+from api_keys import ApiKey
 import gzip
 
 logger = logging.getLogger('Atlas')
@@ -71,6 +72,15 @@ class Atlas:
         self.Alerts = Atlas._Alerts(self)
         self.Whitelist = Atlas._Whitelist(self)
         self.MaintenanceWindows = Atlas._MaintenanceWindows(self)
+        self.ApiKeys = Atlas._ApiKeys(self)
+
+        # In order to get our own Org Id, need to call the Group endpoint for more information
+        uri = Settings.api_resources["Project"]["Get One Project"].format(GROUP_ID=self.group)
+        response = self.network.get(Settings.BASE_URL + uri)
+        group_info = response
+        self.org_id: str = group_info.get('orgId', None)
+        self.cluster_count: int = group_info.get('clusterCount', 0)
+        self.name: str = group_info.get('name', None)
 
     class _Clusters:
         """Clusters API
@@ -309,6 +319,12 @@ class Atlas:
             :return: dict: the new cluster configuration dict
 
             """
+            # Check new_cluster_size type:
+            try:
+                assert isinstance(new_cluster_size,InstanceSizeName)
+            except AssertionError:
+                raise TypeError(f'new_cluster_size must be an instance of InstanceSizeName, not a {type(new_cluster_size)}')
+
             # First check to see if this is a new size, and if the cluster exists
             try:
                 existing_config = self.get_single_cluster_as_obj(cluster=cluster)
@@ -666,7 +682,7 @@ class Atlas:
                                                                date_to=date_to)
                 except Exception as e:
                     raise e
-                each_host.add_log_file(name=log_name,file=log_file)
+                each_host.add_log_file(name=log_name, file=log_file)
                 yield each_host
 
         def get_logs_for_cluster(self,
@@ -697,11 +713,9 @@ class Atlas:
                                                                date_to=date_to)
                 except Exception as e:
                     raise e
-                each_host.add_log_file(name=log_name,file=log_file)
+                each_host.add_log_file(name=log_name, file=log_file)
                 yield each_host
 
-
-
         def _get_measurement_for_host(self, host_obj: Host, granularity: AtlasGranularities = AtlasGranularities.HOUR,
                                       period: AtlasPeriods = AtlasPeriods.WEEKS_1,
                                       measurement: AtlasMeasurementTypes = AtlasMeasurementTypes.Cache.dirty,
@@ -1283,6 +1297,68 @@ class Atlas:
             output: bool = self._update_maint_window(new_config=new_config)
             return output
 
+    class _ApiKeys:
+        """Atlas API Keys API
+
+             see: https://docs.atlas.mongodb.com/reference/api/apiKeys/
+
+             The API Keys resource provides tools to create and maintain Organization and Project Scoped API keys,
+             as well as managing access whitelists.
+
+             Args:
+                 atlas (Atlas): Atlas instance
+             """
+
+        def __init__(self, atlas):
+            self.atlas = atlas
+
+        def _get_api_keys(self) -> List[dict]:
+            """Get all apikeys for the group/project.
+
+            url: https://docs.atlas.mongodb.com/reference/api/projectApiKeys/get-all-apiKeys-in-one-project/
+
+            Returns:
+                List[dict]: A list of dicts, direct payload from the Atlas API
+            """
+            uri = Settings.api_resources["Project API Keys"]["Get All API Keys Assigned to Project"].format(
+                GROUP_ID=self.atlas.group)
+            response = self.atlas.network.get(Settings.BASE_URL + uri)
+            return response['results']
+
+        @property
+        def all_keys(self) -> Iterator[ApiKey]:
+            key_list = self._get_api_keys()
+            for each_entry in key_list:
+                yield ApiKey.fill_from_dict(each_entry)
+
+        def _get_api_key(self, key_id: str) -> dict:
+            """Gets a single apikey
+
+            Args:
+                key_id: the key to be retrieved.
+
+            Returns: dict: single key in the form of the raw atlas response.
+
+            """
+            uri = Settings.api_resources["Organization API Keys"]["Get one Organization API Key"].format(
+                ORG_ID=self.atlas.org_id, API_KEY_ID=key_id)
+            response = self.atlas.network.get(Settings.BASE_URL + uri)
+            return response
+
+        def get_single_key(self, key_id: str) -> ApiKey:
+            """Retrieves a single key
+
+            Args:
+                key_id:
+            """
+            return ApiKey.fill_from_dict(self._get_api_key(key_id))
+
+        def _get_whitelist_entry_for_key(self, key_id: str):
+            uri = Settings.api_resources["Organization API Keys"]["Get Whitelists for API Key"].format(
+                ORG_ID=self.atlas.org_id, API_KEY_ID=key_id)
+            response = self.atlas.network.get(Settings.BASE_URL + uri)
+            return response
+
 
 class AtlasPagination:
     """Atlas Pagination Generic Implementation
